% Figure 5 
% IRFs 
%
clear all;close all;clc;

% Setting up 
countries  = {'DE' , 'IT'} ;
maturities = {'Y02','Y05','Y10','Y30'};
H=30; % max horizon
nw=60; % Newey-West truncation param

% Load data
load('./Data/DemandShock.mat')
load('./Data/GovYields.mat')


%% Estimation
for i=1:length(countries)
cty=countries{i};
% Select estimation sample and outcome variables
if      strcmp(cty,'DE')
startsmpl=datefind(datetime('19-Feb-1999'),DemandShock.Date);endsmpl=datefind(datetime('29-Dec-2017'),DemandShock.Date);
Variables = table(GovYields.Date,GovYields.BMBD02YRY,GovYields.BMBD05YRY,GovYields.BMBD10YRY,GovYields.BMBD30YRY);
Variables.Properties.VariableNames = {'Date','Y02','Y05','Y10','Y30'};
elseif  strcmp(cty,'IT') 
startsmpl=datefind(datetime('31-Aug-2009'),DemandShock.Date);endsmpl=datefind(datetime('29-Dec-2017'),DemandShock.Date);
Variables = table(GovYields.Date,GovYields.BMIT02YRY,GovYields.BMIT05YRY,GovYields.BMIT10YRY,GovYields.BMIT30YRY);
Variables.Properties.VariableNames = {'Date','Y02','Y05','Y10','Y30'};
end
x = DemandShock.([cty,'_D'])(startsmpl:endsmpl);
    for j=1:4 %for each maturity
    mat = maturities{j}; 
    y = Variables.(mat)(startsmpl:endsmpl).*100;  % Scaled to have IRFs in bps
                    for h=0:H-1 %for each horizon
                        yy=y(h+2:end,:)- y(1:end-1-h,:);
                        xx = [ones(size(yy)), x(2:end-h,:)]; 
                        [~,nwse,coeff]=hac(xx,yy,'intercept',false,'display','off','bandwidth',nw);
                        IRF.(cty).(mat)(:,:,h+1) = coeff(2);
                        IRFse.(cty).(mat)(1,:,h+1)=[coeff(2)-1.645*nwse(2),coeff(2)+1.645*nwse(2)]';
                        disp([cty, ' coefficients estimated for maturity ',mat,', for h=',num2str(h+1) ])
                    end%end horizon h
    end %
end % 


%% Plot
close all;
xlab=1:H;
% DE 
figure1=figure('units','normalized','outerposition',[0 0 1 1]);
subplot(2,2,1)
        ax = gca;
        outerpos = ax.OuterPosition;
        ti = ax.TightInset; 
        left = outerpos(1) + ti(1);
        bottom = outerpos(2) + ti(2);
        ax_width = outerpos(3) - ti(1) - ti(3);
        ax_height = outerpos(4) - ti(2) - ti(4);
        ax.Position = [left bottom ax_width ax_height];
    plotdata=squeeze(IRF.DE.Y02) ;
    plot(xlab,plotdata(:,1),'k','LineWidth',3) ;hold on;
    plot(xlab,zeros(H,1),'k:','LineWidth',0.3);
    plotdata=squeeze(IRFse.DE.Y02) ;
    shade(xlab,plotdata(1,:),xlab,plotdata(2,:),'FillType',[1 2;2 1],'FillColor',[0 0 0; 0 0 0],'FillAlpha',0.2);
    hold off;axis tight;ylim([-6 2]);set(gca,'FontSize',20)
    xlabel('days', 'FontSize', 22);ylabel('basis points', 'FontSize', 22);title('2-year maturity', 'FontSize', 24);
 subplot(2,2,2)
        ax = gca;
        outerpos = ax.OuterPosition;
        ti = ax.TightInset; 
        left = outerpos(1) + ti(1);
        bottom = outerpos(2) + ti(2);
        ax_width = outerpos(3) - ti(1) - ti(3);
        ax_height = outerpos(4) - ti(2) - ti(4);
        ax.Position = [left bottom ax_width ax_height];
    plotdata=squeeze(IRF.DE.Y05) ;
    plot(xlab,plotdata(:,1),'k','LineWidth',3) ;hold on;
    plot(xlab,zeros(H,1),'k:','LineWidth',0.3);
    plotdata=squeeze(IRFse.DE.Y05);
    shade(xlab,plotdata(1,:),xlab,plotdata(2,:),'FillType',[1 2;2 1],'FillColor',[0 0 0;0 0 0],'FillAlpha',0.2);
    hold off;axis tight;ylim([-6 2]);set(gca,'FontSize',20)
    xlabel('days', 'FontSize', 22);ylabel('basis points', 'FontSize', 22);title('5-year maturity', 'FontSize', 24);
subplot(2,2,3)
        ax = gca;
        outerpos = ax.OuterPosition;
        ti = ax.TightInset; 
        left = outerpos(1) + ti(1);
        bottom = outerpos(2) + ti(2);
        ax_width = outerpos(3) - ti(1) - ti(3);
        ax_height = outerpos(4) - ti(2) - ti(4);
        ax.Position = [left bottom ax_width ax_height];
    plotdata=squeeze(IRF.DE.Y10) ;
    plot(xlab,plotdata(:,1),'k','LineWidth',3) ;hold on;
    plot(xlab,zeros(H,1),'k:','LineWidth',0.3);
    plotdata=squeeze(IRFse.DE.Y10) ;
    shade(xlab,plotdata(1,:),xlab,plotdata(2,:),'FillType',[1 2;2 1],'FillColor',[0 0 0;0 0 0],'FillAlpha',0.2);
    hold off;axis tight;ylim([-6 2]);set(gca,'FontSize',20)
    xlabel('days', 'FontSize', 22);ylabel('basis points', 'FontSize', 22);title('10-year maturity', 'FontSize', 24);
 subplot(2,2,4)
        ax = gca;
        outerpos = ax.OuterPosition;
        ti = ax.TightInset; 
        left = outerpos(1) + ti(1);
        bottom = outerpos(2) + ti(2);
        ax_width = outerpos(3) - ti(1) - ti(3);
        ax_height = outerpos(4) - ti(2) - ti(4);
        ax.Position = [left bottom ax_width ax_height];
    plotdata=squeeze(IRF.DE.Y30);
    plot(xlab,plotdata(:,1),'k','LineWidth',3) ;hold on;
    plot(xlab,zeros(H,1),'k:','LineWidth',0.3);
    plotdata=squeeze(IRFse.DE.Y30);
    shade(xlab,plotdata(1,:),xlab,plotdata(2,:),'FillType',[1 2;2 1],'FillColor',[0 0 0;0 0 0],'FillAlpha',0.2);
    hold off;axis tight;ylim([-6 2]);set(gca,'FontSize',20)
    xlabel('days', 'FontSize', 22);ylabel('basis points', 'FontSize', 22);title('30-year maturity', 'FontSize', 24);
    
% IT     
figure2=figure('units','normalized','outerposition',[0 0 1 1]);
subplot(2,2,1)
        ax = gca;
        outerpos = ax.OuterPosition;
        ti = ax.TightInset; 
        left = outerpos(1) + ti(1);
        bottom = outerpos(2) + ti(2);
        ax_width = outerpos(3) - ti(1) - ti(3);
        ax_height = outerpos(4) - ti(2) - ti(4);
        ax.Position = [left bottom ax_width ax_height];
    plotdata=squeeze(IRF.IT.Y02) ;
    plot(xlab,plotdata(:,1),'k','LineWidth',3) ;hold on;
    plot(xlab,zeros(H,1),'k:','LineWidth',0.3);
    plotdata=squeeze(IRFse.IT.Y02) ;
    shade(xlab,plotdata(1,:),xlab,plotdata(2,:),'FillType',[1 2;2 1],'FillColor',[0 0 0; 0 0 0],'FillAlpha',0.2);
    hold off;axis tight;ylim([-15 15]);set(gca,'FontSize',20)
    xlabel('days', 'FontSize', 22);ylabel('basis points', 'FontSize', 22);title('2-year maturity', 'FontSize', 24);
 subplot(2,2,2)
        ax = gca;
        outerpos = ax.OuterPosition;
        ti = ax.TightInset; 
        left = outerpos(1) + ti(1);
        bottom = outerpos(2) + ti(2);
        ax_width = outerpos(3) - ti(1) - ti(3);
        ax_height = outerpos(4) - ti(2) - ti(4);
        ax.Position = [left bottom ax_width ax_height];
    plotdata=squeeze(IRF.IT.Y05) ;
    plot(xlab,plotdata(:,1),'k','LineWidth',3) ;hold on;
    plot(xlab,zeros(H,1),'k:','LineWidth',0.3);
    plotdata=squeeze(IRFse.IT.Y05);
    shade(xlab,plotdata(1,:),xlab,plotdata(2,:),'FillType',[1 2;2 1],'FillColor',[0 0 0;0 0 0],'FillAlpha',0.2);
    hold off;axis tight;ylim([-15 15]);set(gca,'FontSize',20)
    xlabel('days', 'FontSize', 22);ylabel('basis points', 'FontSize', 22);title('5-year maturity', 'FontSize', 24);
subplot(2,2,3)
        ax = gca;
        outerpos = ax.OuterPosition;
        ti = ax.TightInset; 
        left = outerpos(1) + ti(1);
        bottom = outerpos(2) + ti(2);
        ax_width = outerpos(3) - ti(1) - ti(3);
        ax_height = outerpos(4) - ti(2) - ti(4);
        ax.Position = [left bottom ax_width ax_height];
    plotdata=squeeze(IRF.IT.Y10) ;
    plot(xlab,plotdata(:,1),'k','LineWidth',3) ;hold on;
    plot(xlab,zeros(H,1),'k:','LineWidth',0.3);
    plotdata=squeeze(IRFse.IT.Y10) ;
    shade(xlab,plotdata(1,:),xlab,plotdata(2,:),'FillType',[1 2;2 1],'FillColor',[0 0 0;0 0 0],'FillAlpha',0.2);
    hold off;axis tight;ylim([-15 15]);set(gca,'FontSize',20)
    xlabel('days', 'FontSize', 22);ylabel('basis points', 'FontSize', 22);title('10-year maturity', 'FontSize', 24);
 subplot(2,2,4)
        ax = gca;
        outerpos = ax.OuterPosition;
        ti = ax.TightInset; 
        left = outerpos(1) + ti(1);
        bottom = outerpos(2) + ti(2);
        ax_width = outerpos(3) - ti(1) - ti(3);
        ax_height = outerpos(4) - ti(2) - ti(4);
        ax.Position = [left bottom ax_width ax_height];
    plotdata=squeeze(IRF.IT.Y30);
    plot(xlab,plotdata(:,1),'k','LineWidth',3) ;hold on;
    plot(xlab,zeros(H,1),'k:','LineWidth',0.3);
    plotdata=squeeze(IRFse.IT.Y30);
    shade(xlab,plotdata(1,:),xlab,plotdata(2,:),'FillType',[1 2;2 1],'FillColor',[0 0 0;0 0 0],'FillAlpha',0.2);
    hold off;axis tight;ylim([-15 15]);set(gca,'FontSize',20)
    xlabel('days', 'FontSize', 22);ylabel('basis points', 'FontSize', 22);title('30-year maturity', 'FontSize', 24);