% Figure 6, 7, 8 
% Reaction of the yield curve:
% Linear and state dependent specification 
%
clear all;close all;clc;

% Select estimation
doregime=2; %0: Linear, 1: Fin. Stress, 2: Negative/Positive

% Load data
load('./Data/DemandShock.mat')
load('./Data/GovYields.mat')
load('./Data/CLIFS.mat')

countries  = {'DE' , 'IT'} ;
shockmaturities = {'short','long'};
nw = 60;

CLIFS.DEstress=CLIFS.prctileDE>70;
CLIFS.ITstress=CLIFS.prctileIT>70;
DemandShock.IT_short = DemandShock.IT_2Y;
DemandShock.IT_long = DemandShock.IT_10Y;


%% Estimation
%0:Linear
if doregime==0
for i=1:length(countries)
cty=countries{i};
% Short and long shocks
for s = 1:2
shock = shockmaturities{s};
%Select country dependent variables
if      strcmp(cty,'DE')
startsmpl=datefind(datetime('19-Feb-1999'),DemandShock.Date);endsmpl=datefind(datetime('29-Dec-2017'),DemandShock.Date);
Variables = table(GovYields.Date,GovYields.BMBD02YRY,GovYields.BMBD03YRY,GovYields.BMBD05YRY,GovYields.BMBD07YRY,GovYields.BMBD10YRY,GovYields.BMBD20YRY,GovYields.BMBD30YRY);
Variables.Properties.VariableNames = {'Date','Y02','Y03','Y05','Y07','Y10','Y20','Y30'};
elseif  strcmp(cty,'IT') 
startsmpl=datefind(datetime('31-Aug-2009'),DemandShock.Date);
    if strcmp(shock,'short')
    startsmpl=datefind(datetime('18-Oct-2010'),DemandShock.Date);
    end
endsmpl=datefind(datetime('29-Dec-2017'),DemandShock.Date);
Variables = table(GovYields.Date,GovYields.BMIT02YRY,GovYields.BMIT03YRY,GovYields.BMIT05YRY,GovYields.BMIT07YRY,GovYields.BMIT10YRY,GovYields.BMIT15YRY,GovYields.BMIT30YRY);
Variables.Properties.VariableNames = {'Date','Y02','Y03','Y05','Y07','Y10','Y15','Y30'};
end
x = DemandShock.([cty,'_',shock])(startsmpl:endsmpl);
    for j=2:width(Variables) %for each maturity
    var = Variables.Properties.VariableNames{j}; 
    y = Variables.(var)(startsmpl:endsmpl).*100;  % Scaled to have IRFs in bps
    yy=y(2:end,:)-y(1:end-1,:);
    xx = [ones(size(yy)), x(2:end,:)]; 
    [~,nwse,coeff]=hac(xx,yy,'intercept',false,'display','off','bandwidth',nw);
    COEFF.(cty).(shock).(var)(:,:,1) = coeff(2);
    SE.(cty).(shock).(var)(1,:,1)=[coeff(2)-1.645*nwse(2),coeff(2)+1.645*nwse(2)]';
    disp([cty,' ',shock, ' coefficients estimated at maturity ',var])
    end % variable
   end % shock maturity
  end %country
end% if doregime=0

%1,2: Non-linear
if doregime>0
for i=1:length(countries)    
cty=countries{i};
% Short and long shocks
for s = 1:2
shock = shockmaturities{s};
%Select country dependent variables
if      strcmp(cty,'DE')
startsmpl=datefind(datetime('19-Feb-1999'),DemandShock.Date);endsmpl=datefind(datetime('29-Dec-2017'),DemandShock.Date);
Variables = table(GovYields.Date,GovYields.BMBD02YRY,GovYields.BMBD03YRY,GovYields.BMBD05YRY,GovYields.BMBD07YRY,GovYields.BMBD10YRY,GovYields.BMBD20YRY,GovYields.BMBD30YRY);
Variables.Properties.VariableNames = {'Date','Y02','Y03','Y05','Y07','Y10','Y20','Y30'};
elseif  strcmp(cty,'IT') 
startsmpl=datefind(datetime('31-Aug-2009'),DemandShock.Date);endsmpl=datefind(datetime('29-Dec-2017'),DemandShock.Date);
Variables = table(GovYields.Date,GovYields.BMIT02YRY,GovYields.BMIT03YRY,GovYields.BMIT05YRY,GovYields.BMIT07YRY,GovYields.BMIT10YRY,GovYields.BMIT15YRY,GovYields.BMIT30YRY);
Variables.Properties.VariableNames = {'Date','Y02','Y03','Y05','Y07','Y10','Y15','Y30'};
end
x = DemandShock.([cty,'_',shock])(startsmpl:endsmpl);
% Specify regime indicator function
if doregime==1
D = CLIFS.([cty,'stress'])(startsmpl:endsmpl);;  
elseif doregime==2
D = x<0;
else
    error('Incorrect regime specification')
end
    for j=2:width(Variables) %for each maturity
    var = Variables.Properties.VariableNames{j}; 
    y = Variables.(var)(startsmpl:endsmpl).*100;  % Scaled to have IRFs in bps
    yy=y(2:end,:)-y(1:end-1,:);
    DD = D(2:end,:);
    xx = [ones(size(yy)), x(2:end,:)]; 
    xx = [DD.*xx , (1-DD).*xx];  %[D=1 stress\neg, D=0 non-stress\pos] 
    [~,nwse,coeff]=hac(xx,yy,'intercept',false,'display','off','bandwidth',nw);
    COEFF.(cty).(shock).(var)(:,:,1) = [coeff(2),coeff(4)];
    SE.(cty).(shock).(var)(1,1:2,1)=[coeff(2)-1.645*nwse(2),coeff(2)+1.645*nwse(2)]; %first col: crisis
    SE.(cty).(shock).(var)(1,3:4,1)=[coeff(4)-1.645*nwse(4),coeff(4)+1.645*nwse(4)];%second col: no crisis
    disp([cty,' ',shock, ' coefficients estimated at maturity ',var,' regime',doregime])
    end % variable
   end % shock maturity
  end %country
end% if doregime=0


%% Plot


if doregime==0
%DE
xlab=[0,2,3,5,7,10,20,30];
N=size(xlab,2);
figure1=figure('units','normalized','outerposition',[0 0 1 1]);
subplot(2,1,1)
    plot(xlab, [0;cell2mat(struct2cell(COEFF.DE.short))],'k-o','LineWidth',3 ,'MarkerIndices',2:length(xlab)) ;hold on;ylim([-4 0.5]);set(gca,'FontSize',20)
    plot(xlab,zeros(N,1),'k:','LineWidth',0.4);
    plot(xlab, [0,0;cell2mat(struct2cell(SE.DE.short))], 'k--o','MarkerIndices',2:length(xlab)) ;
    hold off;
    xlabel('maturity','FontSize',20);ylabel('basis points','FontSize',20);title('Short-Term Shock','FontSize',24)
    ax = gca;
    ax.PlotBoxAspectRatio=[0.8 0.244623655913979 0.244623655913979];
subplot(2,1,2)
    plot(xlab, [0;cell2mat(struct2cell(COEFF.DE.long))] ,'k-o','LineWidth',3,'MarkerIndices',2:length(xlab)) ;hold on;ylim([-4 0.5]);set(gca,'FontSize',20)
    plot(xlab,zeros(N,1),'k:','LineWidth',0.4);
    plot(xlab, [0,0;cell2mat(struct2cell(SE.DE.long))], 'k--o','MarkerIndices',2:length(xlab)) ;
    xlabel('maturity','FontSize',20);ylabel('basis points','FontSize',20);title('Long-Term Shock','FontSize',24);
    ax = gca;
    ax.PlotBoxAspectRatio=[0.8 0.244623655913979 0.244623655913979];
    hold off;
%IT
xlab=[0,2,3,5,7,10,15,30];
figure2=figure('units','normalized','outerposition',[0 0 1 1]);
subplot(2,1,1)
    plot(xlab, [0;cell2mat(struct2cell(COEFF.IT.short))],'k-o','LineWidth',3 ,'MarkerIndices',2:length(xlab)) ;hold on;ylim([-6 0.5]);set(gca,'FontSize',20)
    plot(xlab,zeros(N,1),'k:','LineWidth',0.4);
    plot(xlab, [0,0;cell2mat(struct2cell(SE.IT.short))], 'k--o','MarkerIndices',2:length(xlab)) ;
    hold off;
    xlabel('maturity','FontSize',20);ylabel('basis points','FontSize',20);title('Short-Term shock','FontSize',24)
    ax = gca;
    ax.PlotBoxAspectRatio=[0.8 0.244623655913979 0.244623655913979];
subplot(2,1,2)
    plot(xlab, [0;cell2mat(struct2cell(COEFF.IT.long))] ,'k-o','LineWidth',3,'MarkerIndices',2:length(xlab)) ;hold on;ylim([-6 0.5]);set(gca,'FontSize',20)
    plot(xlab,zeros(N,1),'k:','LineWidth',0.4);
    plot(xlab, [0,0;cell2mat(struct2cell(SE.IT.long))], 'k--o','MarkerIndices',2:length(xlab)) ;
    xlabel('maturity','FontSize',20);ylabel('basis points','FontSize',20);title('Long-Term Shock','FontSize',24)
    hold off;
    ax = gca;
    ax.PlotBoxAspectRatio=[0.8 0.244623655913979 0.244623655913979];
    
    
    
  
elseif doregime==1
%DE
xlab=[0,2,3,5,7,10,20,30];
N=size(xlab,2);
figure1=figure('units','normalized','outerposition',[0 0 1 1]);
subplot(2,1,1)
    plotdata=[0,0;cell2mat(struct2cell(COEFF.DE.short))] ;%[HighStress,LowStress];
    plot(xlab,plotdata(:,1),'k-o','LineWidth',2.3,'MarkerIndices',2:length(xlab),'MarkerSize',10) ;hold on; ylim([-4 0.5]);set(gca,'FontSize',20)
    plot(xlab,plotdata(:,2),'k-*','LineWidth',2.3,'MarkerIndices',2:length(xlab),'MarkerSize',10,'Color',[.5,.5,.5]) ;plot(xlab,zeros(N,1),'k:','LineWidth',0.1);
    plotdata=[0 0,0 0;cell2mat(struct2cell(SE.DE.short))];
    shade(xlab,plotdata(:,1),xlab,plotdata(:,2),'FillType',[1 2;2 1],'FillColor',[0 0 0;0 0 0],'FillAlpha',0.45);
    shade(xlab,plotdata(:,3),xlab,plotdata(:,4),'FillType',[1 2;2 1],'FillColor',[0 0 0;0 0 0],'FillAlpha',0.15);
    hold off;
    legend({'High Stress','No Stress'},'FontSize',18);xlabel('maturity','FontSize',20);ylabel('basis points','FontSize',20);title('Short-Term Shock','FontSize',24)
    ax = gca;    
    ax.PlotBoxAspectRatio=[0.8 0.244623655913979 0.244623655913979];
subplot(2,1,2)
    plotdata=[0,0;cell2mat(struct2cell(COEFF.DE.long))] ;
    plot(xlab,plotdata(:,1),'k-o','LineWidth',2.3,'MarkerIndices',2:length(xlab),'MarkerSize',10) ;hold on; ylim([-4 0.5]);set(gca,'FontSize',20)
    plot(xlab,plotdata(:,2),'k-*','LineWidth',2.3,'MarkerIndices',2:length(xlab),'MarkerSize',10,'Color',[.5,.5,.5]) ;plot(xlab,zeros(N,1),'k:','LineWidth',0.1);
    plotdata=[0 0,0 0;cell2mat(struct2cell(SE.DE.long))];
    shade(xlab,plotdata(:,1),xlab,plotdata(:,2),'FillType',[1 2;2 1],'FillColor',[0 0 0;0 0 0],'FillAlpha',0.45);
    shade(xlab,plotdata(:,3),xlab,plotdata(:,4),'FillType',[1 2;2 1],'FillColor',[0 0 0;0 0 0],'FillAlpha',0.15);
    hold off;
    legend({'High Stress','No Stress'},'FontSize',18);xlabel('maturity','FontSize',20);ylabel('basis points','FontSize',20);title('Long-Term Shock','FontSize',24)
    ax = gca;
    ax.PlotBoxAspectRatio=[0.8 0.244623655913979 0.244623655913979];
    
%IT
xlab=[0,2,3,5,7,10,15,30];
figure2=figure('units','normalized','outerposition',[0 0 1 1]);
subplot(2,1,1)
    plotdata=[0,0;cell2mat(struct2cell(COEFF.IT.short))] ;
    plot(xlab,plotdata(:,1),'k-o','LineWidth',2.3,'MarkerIndices',2:length(xlab),'MarkerSize',10) ;hold on;ylim([-8 0.5]);set(gca,'FontSize',20)
    plot(xlab,plotdata(:,2),'k-*','LineWidth',2.3,'MarkerIndices',2:length(xlab),'MarkerSize',10,'Color',[.5,.5,.5]) ;plot(xlab,zeros(N,1),'k:','LineWidth',0.1);
    plotdata=[0 0,0 0;cell2mat(struct2cell(SE.IT.short))];
    shade(xlab,plotdata(:,1),xlab,plotdata(:,2),'FillType',[1 2;2 1],'FillColor',[0 0 0;0 0 0],'FillAlpha',0.45);
    shade(xlab,plotdata(:,3),xlab,plotdata(:,4),'FillType',[1 2;2 1],'FillColor',[0 0 0;0 0 0],'FillAlpha',0.15);
    hold off;
    legend({'High Stress','No Stress'},'FontSize',18);xlabel('maturity','FontSize',20);ylabel('basis points','FontSize',20);title('Short-Term shock','FontSize',24)
    ax = gca;
    ax.PlotBoxAspectRatio=[0.8 0.244623655913979 0.244623655913979];
subplot(2,1,2)
    plotdata=[0,0;cell2mat(struct2cell(COEFF.IT.long))] ;
    plot(xlab,plotdata(:,1),'k-o','LineWidth',2.3,'MarkerIndices',2:length(xlab),'MarkerSize',10) ;hold on; ylim([-8 0.5]);set(gca,'FontSize',20)
    plot(xlab,plotdata(:,2),'k-*','LineWidth',2.3,'MarkerIndices',2:length(xlab),'MarkerSize',10,'Color',[.5,.5,.5]) ;plot(xlab,zeros(N,1),'k:','LineWidth',0.1);
    plotdata=[0 0,0 0;cell2mat(struct2cell(SE.IT.long))];
    shade(xlab,plotdata(:,1),xlab,plotdata(:,2),'FillType',[1 2;2 1],'FillColor',[0 0 0;0 0 0],'FillAlpha',0.45);
    shade(xlab,plotdata(:,3),xlab,plotdata(:,4),'FillType',[1 2;2 1],'FillColor',[0 0 0;0 0 0],'FillAlpha',0.15);
    hold off;
    legend({'High Stress','No Stress'},'FontSize',18);xlabel('maturity','FontSize',20);ylabel('basis points','FontSize',20);title('Long-Term Shock','FontSize',24)
    ax = gca;
    ax.PlotBoxAspectRatio=[0.8 0.244623655913979 0.244623655913979];
    
    
    
elseif doregime==2
%DE    
 xlab=[0,2,3,5,7,10,20,30];
 N=size(xlab,2);
figure1=figure('units','normalized','outerposition',[0 0 1 1]);
subplot(2,1,1)
    plotdata=[0,0;cell2mat(struct2cell(COEFF.DE.short))] ; %[Negative,Positive]
    plot(xlab,plotdata(:,1),'k-s','LineWidth',2.3,'MarkerIndices',2:length(xlab),'MarkerSize',10) ;hold on; ylim([-4 0.5]);set(gca,'FontSize',20)
    plot(xlab,plotdata(:,2),'k-x','LineWidth',2.3,'MarkerIndices',2:length(xlab),'MarkerSize',10,'Color',[.5,.5,.5]) ;plot(xlab,zeros(N,1),'k:','LineWidth',0.1);
    plotdata=[0 0,0 0;cell2mat(struct2cell(SE.DE.short))];
    shade(xlab,plotdata(:,1),xlab,plotdata(:,2),'FillType',[1 2;2 1],'FillColor',[0 0 0;0 0 0],'FillAlpha',0.45);
    shade(xlab,plotdata(:,3),xlab,plotdata(:,4),'FillType',[1 2;2 1],'FillColor',[0 0 0;0 0 0],'FillAlpha',0.15);
    hold off;
    legend({'Negative','Positive'},'FontSize',18);xlabel('maturity','FontSize',20);ylabel('basis points','FontSize',20);title('Short-Term Shock','FontSize',24)
    ax = gca;
    ax.PlotBoxAspectRatio=[0.8 0.244623655913979 0.244623655913979];
subplot(2,1,2)
    plotdata=[0,0;cell2mat(struct2cell(COEFF.DE.long))] ;
    plot(xlab,plotdata(:,1),'k-s','LineWidth',2.3,'MarkerIndices',2:length(xlab),'MarkerSize',10) ;hold on; ylim([-4 0.5]);set(gca,'FontSize',20)
    plot(xlab,plotdata(:,2),'k-x','LineWidth',2.3,'MarkerIndices',2:length(xlab),'MarkerSize',10,'Color',[.5,.5,.5]) ;plot(xlab,zeros(N,1),'k:','LineWidth',0.1);
    plotdata=[0 0,0 0;cell2mat(struct2cell(SE.DE.long))];
    shade(xlab,plotdata(:,1),xlab,plotdata(:,2),'FillType',[1 2;2 1],'FillColor',[0 0 0;0 0 0],'FillAlpha',0.45);
    shade(xlab,plotdata(:,3),xlab,plotdata(:,4),'FillType',[1 2;2 1],'FillColor',[0 0 0;0 0 0],'FillAlpha',0.15);
    hold off;
    legend({'Negative','Positive'},'FontSize',18);xlabel('maturity','FontSize',20);ylabel('basis points','FontSize',20);title('Long-Term Shock','FontSize',24)
    ax = gca;
    ax.PlotBoxAspectRatio=[0.8 0.244623655913979 0.244623655913979];    
    
%IT
 xlab=[0,2,3,5,7,10,15,30];
 N=size(xlab,2);
figure2=figure('units','normalized','outerposition',[0 0 1 1]);
subplot(2,1,1)
    plotdata=[0,0;cell2mat(struct2cell(COEFF.IT.short))] ;
    plot(xlab,plotdata(:,1),'k-s','LineWidth',2.3,'MarkerIndices',2:length(xlab),'MarkerSize',10) ;hold on; ylim([-8 0.5]);set(gca,'FontSize',20)
    plot(xlab,plotdata(:,2),'k-x','LineWidth',2.3,'MarkerIndices',2:length(xlab),'MarkerSize',10,'Color',[.5,.5,.5]) ;plot(xlab,zeros(N,1),'k:','LineWidth',0.1);
    plotdata=[0 0,0 0;cell2mat(struct2cell(SE.IT.short))];
    shade(xlab,plotdata(:,1),xlab,plotdata(:,2),'FillType',[1 2;2 1],'FillColor',[0 0 0;0 0 0],'FillAlpha',0.45);
    shade(xlab,plotdata(:,3),xlab,plotdata(:,4),'FillType',[1 2;2 1],'FillColor',[0 0 0;0 0 0],'FillAlpha',0.15);
    %plot(xlab,[plotdata(:,3),plotdata(:,4)],'k:','LineWidth',1.8);
    hold off;
    legend({'Negative','Positive'},'FontSize',18);xlabel('maturity','FontSize',20);ylabel('basis points','FontSize',20);title('Short-Term Shock','FontSize',24)
    ax = gca;
    ax.PlotBoxAspectRatio=[0.8 0.244623655913979 0.244623655913979];
    
subplot(2,1,2)
    plotdata=[0,0;cell2mat(struct2cell(COEFF.IT.long))] ;
    plot(xlab,plotdata(:,1),'k-s','LineWidth',2.3,'MarkerIndices',2:length(xlab),'MarkerSize',10) ;hold on; ylim([-8 0.5]);set(gca,'FontSize',20)
    plot(xlab,plotdata(:,2),'k-x','LineWidth',2.3,'MarkerIndices',2:length(xlab),'MarkerSize',10,'Color',[.5,.5,.5]) ;plot(xlab,zeros(N,1),'k:','LineWidth',0.1);
    plotdata=[0 0,0 0;cell2mat(struct2cell(SE.IT.long))];
    shade(xlab,plotdata(:,1),xlab,plotdata(:,2),'FillType',[1 2;2 1],'FillColor',[0 0 0;0 0 0],'FillAlpha',0.45);
    shade(xlab,plotdata(:,3),xlab,plotdata(:,4),'FillType',[1 2;2 1],'FillColor',[0 0 0;0 0 0],'FillAlpha',0.15);
    %plot(xlab,[plotdata(:,3),plotdata(:,4)],'k:','LineWidth',1.8);
    hold off;
    legend({'Negative','Positive'},'FontSize',18);xlabel('maturity','FontSize',20);ylabel('basis points','FontSize',20);title('Long-Term Shock','FontSize',24)
    ax = gca;
    ax.PlotBoxAspectRatio=[0.8 0.244623655913979 0.244623655913979];
end