% Table 2, 3, 4, 5 
% Treasury yields and CDS spreads 
%
clear all;close all;clc;
warning off

% Setting up 
countries  = {'DE' , 'IT'} ;
nw=60; % Newey-West truncation param

% Select estimation
doregime=2; %0: Linear, 1: Fin. Stress, 2: Negative/Positive


% Load data
load('./Data/DemandShock.mat')
load('./Data/GovYields.mat')
load('./Data/CDS')
load('./Data/Equity.mat')
load('./Data/BBEAI.mat')
load('./Data/CLIFS.mat')


%Table 2 variables
Variables = table(GovYields.Date,GovYields.BMBD02YRY,GovYields.BMBD05YRY,GovYields.BMBD10YRY,GovYields.BMBD30YRY);
Variables = addvars(Variables,GovYields.BMIT02YRY,GovYields.BMIT05YRY,GovYields.BMIT10YRY,GovYields.BMIT30YRY);
Variables = addvars(Variables,GovYields.BMFR02YRY,GovYields.BMFR05YRY,GovYields.BMFR10YRY,GovYields.BMFR30YRY);
Variables = addvars(Variables,GovYields.BMES02YRY,GovYields.BMES05YRY,GovYields.BMES10YRY,GovYields.BMES30YRY);
Variables = addvars(Variables,GovYields.BMNL02YRY,GovYields.BMNL05YRY,GovYields.BMNL10YRY,GovYields.BMNL30YRY);
Variables = addvars(Variables,CDS.DEG2EAC,CDS.DEGAEAC,CDS.ITG2EAC,CDS.ITGAEAC,CDS.FRG2EAC ...
    ,CDS.FRGAEAC,CDS.ESG2EAC,CDS.ESGAEAC,CDS.NLG2AC,CDS.NLGAAC   );
Variables.Properties.VariableNames = {'Date','DEY02','DEY05','DEY10','DEY30','ITY02','ITY05','ITY10','ITY30',...
    'FRY02','FRY05','FRY10','FRY30','ESY02','ESY05','ESY10','ESY30','NLY02','NLY05','NLY10','NLY30',...
    'DECDS2Y','DECDS10Y','ITCDS2Y','ITCDS10Y','FRCDS2Y','FRCDS10Y',...
    'ESCDS2Y','ESCDS10Y','NLCDS2Y','NLCDS10Y'};
% Table 3 variables
Variables.DAX=log(Equity.DAX);
Variables.FTSEMIB=log(Equity.FTSEMIB);
Variables.STOXX50=log(Equity.STOXX50);
Variables.CAC40=log(Equity.CAC40);
Variables.IBEX35I=log(Equity.IBEX35I);
Variables.AEX=log(Equity.AEX);
Variables=join(Variables,BBEAI);
Variablenames = Variables.Properties.VariableNames;Variablenames=Variablenames(2:end)';

CLIFS.DEstress=CLIFS.prctileDE>70;
CLIFS.ITstress=CLIFS.prctileIT>70;
%% Estimation
% Preallocate estimate holder
ESTIM = table();


% 0: Linear estimation
if doregime==0
    for i=1:length(countries)
    cty=countries{i};
    % Select estimation sample and outcome variables
    if      strcmp(cty,'DE')
    startsmpl=datefind(datetime('19-Feb-1999'),DemandShock.Date);endsmpl=datefind(datetime('29-Dec-2017'),DemandShock.Date);
    elseif  strcmp(cty,'IT') 
    startsmpl=datefind(datetime('31-Aug-2009'),DemandShock.Date);endsmpl=datefind(datetime('29-Dec-2017'),DemandShock.Date);
    end
    x = DemandShock.([cty,'_D'])(startsmpl:endsmpl);
    for j = 1 : length(Variablenames)
        var = Variablenames{j};    
        y = Variables.(var)(startsmpl:endsmpl).*100;
         yy=y(2:end,:)-y(1:end-1,:);
         xx = [ones(size(yy)),  x(2:end,:)]; 
         [~,nwse,coeff]=hac(xx,yy,'intercept',false,'display','off','bandwidth',nw);
         ESTIM.(['Coeff_',cty])(var)   = coeff(2);
         ESTIM.(['SE_',cty])(var)      = nwse(2);
         disp([cty,' ',var,' coefficient estimated'])
    end %variables
    end % countries

% 1,2: Non-linear estimation
elseif doregime>0
    for i=1:length(countries)
    cty=countries{i};
    % Select estimation sample and outcome variables
    if      strcmp(cty,'DE')
    startsmpl=datefind(datetime('19-Feb-1999'),DemandShock.Date);endsmpl=datefind(datetime('29-Dec-2017'),DemandShock.Date);
    elseif  strcmp(cty,'IT') 
    startsmpl=datefind(datetime('31-Aug-2009'),DemandShock.Date);endsmpl=datefind(datetime('29-Dec-2017'),DemandShock.Date);
    end
    x = DemandShock.([cty,'_D'])(startsmpl:endsmpl);
    % Specify regime indicator function
    if doregime==1
    D = CLIFS.([cty,'stress'])(startsmpl:endsmpl);;  
    elseif doregime==2
    D = x<0;
    else
        error('Incorrect regime specification')
    end
    for j = 1 : length(Variablenames)
        var = Variablenames{j};    
        y = Variables.(var)(startsmpl:endsmpl).*100;
         yy=y(2:end,:)-y(1:end-1,:);
         DD = D(2:end,:);
         xx = [ones(size(yy)), x(2:end,:)]; 
         xx = [DD.*xx , (1-DD).*xx];  %[D=1 stress\neg, D=0 non-stress\pos] 
         [coeffCov,nwse,coeff]=hac(xx,yy,'intercept',false,'display','off','bandwidth',nw);
         ESTIM.(['Coeff1_',cty])(var)   = coeff(2); %D=1
         ESTIM.(['SE1_',cty])(var)      = nwse(2);
         ESTIM.(['Coeff0_',cty])(var)   = coeff(4); % D=0
         ESTIM.(['SE0_',cty])(var)      = nwse(4);   
         F = (   (coeff(2)-coeff(4)) /   sqrt(coeffCov(2,2) + coeffCov(4,4) - 2*coeffCov(2,4) )      )^2   ;
         v=length(yy)-size(xx,2);
         ESTIM.(['FtestPval_',cty])(var) =  1-   fcdf( F, 1 , v-1);
         disp([cty,' ',var,' coefficient estimated'])
    end %variables
    end % countries


end % doregime

